# coding=utf-8

import inspect
from optparse import make_option

from django.core.management.base import NoArgsCommand
from django.db.models.base import ModelBase
from django.utils.importlib import import_module
from django.db.models.fields.files import FileDescriptor
from django.conf import settings


class Command(NoArgsCommand):
    """
    Команда обходит все зарегистрированные модели,
    в которых есть поля FileField.
    Если для поля установлен AtcfsStorage, или он установлен глобально,
    то файл переносится на сервер ATCFS.
    """

    option_list = NoArgsCommand.option_list + (
        make_option(
            '--delete',
            action='store_true',
            dest='delete',
            default=False,
            help=u'Удалять файлы вместо перемещения.'
        ),
    )

    def _get_fields(self, model):
        """
        Выбираем в модели все переменные, являющиеся FileField-полями.
        :param model: класс модели
        :return: список названий полей FileField
        """
        fields = []
        for nam, mem in model.__dict__.iteritems():
            if callable(mem):
                continue
            if nam.startswith('_'):
                continue
            if isinstance(mem, FileDescriptor):
                fields.append(nam)
        return fields

    def _get_models(self):
        """
        Берем все модели, в которых есть FileField
        :return: список классов моделей
        """
        models = {}

        is_model = lambda x: (
            inspect.isclass(x) and
            isinstance(x, ModelBase) and
            not x._meta.abstract
        )

        apps = settings.INSTALLED_APPS
        for app in apps:
            try:
                models_module = import_module('{0}.models'.format(app))
            except ImportError:
                continue
            for name, cls in inspect.getmembers(models_module):
                if is_model(cls) and cls not in models:
                    fields = self._get_fields(cls)
                    if fields:
                        models[cls] = fields

        return models

    def _delete_all_files(self, models):
        """
        Сервисный метод. Используется для технических нужд.
        В работе команды не учавствует.
        """
        for model, fields in models.iteritems():
            kwargs = dict(zip(fields, ['']*len(fields)))
            print '{0}: {1}'.format(model, model.objects.all().update(**kwargs))

    def handle_noargs(self, **options):
        models = self._get_models()
        if options['delete']:
            self._delete_all_files(models)
        else:
            pass
